﻿EnableExplicit
;/------------------
;|
;| Template Version: 3
;|
;|
;| NEW SINCE VERSION 2:
;| Added three Parameters for Template_GetHeader()
;|
;| NEW SINCE VERSION 3:
;| Changed Returnvalues to INTEGER, added #USE_UNICODE and added possibility to set more anchors
;|
;/------------------

#TEMPLATE_VERSION = 3
;If i change anything with the procedures (add/delete Parameters or anything like this),
;the version may increment.
;This is to make sure, there will be no Stack-corruption and the template will not be used,
;till it gets updated to the new format.

CompilerIf #PB_Compiler_Unicode
	#USE_UNICODE = $8000
CompilerElse
	#USE_UNICODE = 0
CompilerEndIf

ProcedureDLL AttachProcess(Instance)
	;Called, when dll is loaded.
	;We do nothing but initialize our Global String-Variable
	Global HTML.s, LineNum.i
	
EndProcedure

ProcedureDLL DetachProcess(Instance)
	;Called, when dll is unloaded.
	;Do you have anything to free?

	;No...
EndProcedure

Procedure.s Internal_DoHTML(CodeLine.s)
	;This procedure changes the CodeLine to make it "html-ready"
	Protected i, Add.s

	i = FindString(CodeLine, Chr(3))
	If i
		;Chr(3) will be used as a marker.
		;Anything before Chr(3) will be treated as html-code and should not get changed
		Add      = Left(CodeLine, i - 1)
		CodeLine = Mid(CodeLine, i + 1)
	EndIf
	CodeLine = ReplaceString(ReplaceString(CodeLine, "&", "&amp;"), " ", "&nbsp;")
	CodeLine = ReplaceString(CodeLine, #TAB$, "&nbsp;&nbsp;")
	CodeLine = ReplaceString(ReplaceString(CodeLine, "(", "&#40;"), ")", "&#41;")
	CodeLine = ReplaceString(ReplaceString(CodeLine, "<", "&lt;"), ">", "&gt;")
	CodeLine = ReplaceString(ReplaceString(CodeLine, Chr(34), "&quot;"), ":", "&#58;")
	CodeLine = RemoveString(CodeLine, Chr($EF) + Chr($BB) + Chr($BF))

	ProcedureReturn Add + CodeLine
EndProcedure

ProcedureDLL Template_Init(Needed_Version)
	;When template-DLL has been successfully loaded,
	;HistoryViewer will call immediately Template_Init() to check
	;if this template has at least the Needed_Version.
	;
	;This is to check, if this version will work together with history viewer.
	;If Returnvalue & 1 <> #True, it will be unloaded immediately.
	Protected Result
	
	If Needed_Version <= #TEMPLATE_VERSION
		Result = #True
	EndIf
	
	ProcedureReturn Result | #USE_UNICODE
EndProcedure

ProcedureDLL Template_GetEmptySite()
	;The returned String will contain all of the html-code for displaying an empty site.
	
	HTML = "<html><head><title></title><style type='text/css'>" + #CRLF$
	HTML + "body { background-color: #E5E5E5;scrollbar-face-color: #BDD0DF;scrollbar-highlight-color: #EFF5F9;"
	HTML + "scrollbar-shadow-color: #BDD0DF;scrollbar-3dlight-color: #D1D7DC;scrollbar-arrow-color:  #000000;"
	HTML + "scrollbar-track-color: #EFEFEF;scrollbar-darkshadow-color: #98AAB1; }" + #CRLF$
	HTML + "</style></head><body>&nbsp;</body></html>"
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_GetHeader(FileName.s, Title.s, Version, Date, BOM, ShowDifferences, User.s, LinesAdded, LinesDeleted)
	;Called, whenever an outputrequest has been made.
	;This procedure should return the pointer to a string, which contains anything necessary for a header.
	
	;You have a few Variables, which you can put into the html-code:
	;FileName:        This is the filename, the user selected.
	;Title:           User can set special title to versions.
	;                 This is given to you as link! and looks like this: [Title], or [] when emtpy.
	;Version:         The version-number of the shown history
	;Date:            Timestamp of the version
	;BOM:             if #PB_UTF8 you should set the charset of your output correctly
	;ShowDifferences: If <> 0 user activated the Code-Difference-View.
	;                 You may alter your header.
	;---Added since Version 2
	;User:            The Username, who made the changes (only used with postgresql)
	;LinesAdded:      The number of lines which have been added
	;LinesDeleted:    The number of lines which have been deleted
	
	LineNum = 1
	HTML = "<html><head><title></title>" + #CRLF$
	HTML + "<meta http-equiv='content-type' content='text/html; charset="
	If BOM = #PB_UTF8
		HTML + "utf-8"
	Else
		HTML + "ISO-8859-1"
	EndIf
	HTML + "' />" + #CRLF$
	HTML + "<style type='text/css'>" + #CRLF$
	HTML + "body { background-color: #E5E5E5;scrollbar-face-color: #BDD0DF;scrollbar-highlight-color: #EFF5F9;"
	HTML + "font-family: 'Times New Roman', Verdana, Helvetica, Arial, sans-serif;"
	HTML + "scrollbar-shadow-color: #BDD0DF;scrollbar-3dlight-color: #D1D7DC;scrollbar-arrow-color:  #000000;"
	HTML + "scrollbar-track-color: #EFEFEF;scrollbar-darkshadow-color: #98AAB1; }" + #CRLF$
	HTML + ".code { font-family: Courier, 'Courier New', sans-serif; font-size: 11px; color: #006600;"
	HTML + "background-color: #FAFAFA }" + #CRLF$
	HTML + ".code_del { font-family: Courier, 'Courier New', sans-serif; font-size: 11px; color: #000000;background-color: #ff8A8A; }" + #CRLF$
	HTML + ".code_add { font-family: Courier, 'Courier New', sans-serif; font-size: 11px; color: #000000;background-color: #8AFF8A; }" + #CRLF$
	HTML + ".filename { font-weight: bold; font-size: 20px; color: #CA2C00; }" + #CRLF$
	HTML + ".vars { font-weight: bold; color: #043865; }" + #CRLF$
	HTML + ".fln { border-right-color: #dedede; border-right-style: solid; border-right-width: 5px; font-family: Courier, 'Courier New', sans-serif; font-size: 11px; background-color: #eeeeee; color: #505050; text-align: right; }" + #CRLF$
	HTML + "a:link { color: #280CA8; }" + #CRLF$
	HTML + "a:visited { color: #280CA8; }" + #CRLF$
	HTML + "a:hover { color: #22B2DD; background-color: #800080; text-decoration: none; }" + #CRLF$
	HTML + "a:active { color: #280CA8; }" + #CRLF$
	HTML + "</style>" + #CRLF$
	HTML + "</head>" + #CRLF$
	HTML + "<body>" + #CRLF$
	HTML + "<div class='filename'>" + FileName + "</div><br>" + #CRLF$
	HTML + "<table style='font-size:16px'><tr>" + #CRLF$
	HTML + "<td>Version</td><td class='vars'>#" + Str(Version) + " " + Title + "</td></tr>" + #CRLF$
	If User
		HTML + "<tr><td>Made by:</td><td class='vars'>" + User + "</td></tr>" + #CRLF$
	EndIf
	HTML + "<tr><td>Created:</td><td class='vars'>" + FormatDate("%dd.%mm.%yyyy %hh:%ii:%ss", Date) + "</td></tr>" + #CRLF$
	If ShowDifferences
		HTML + "<tr><td>Lines Added:</td><td class='vars'>" + Str(LinesAdded) + "</td></tr>" + #CRLF$
		HTML + "<tr><td>Lines Deleted:</td><td class='vars'>" + Str(LinesDeleted) + "</td></tr>" + #CRLF$
	EndIf
	HTML + "</table><hr><br>" + #CRLF$
	HTML + "<table align='center' border='0' cellpadding='0' cellspacing='0' width='90%'><tbody>" + #CRLF$
	If ShowDifferences
		HTML + "<tr><td colspan='2' class='code_add'>Added Lines of Code</td></tr>" + #CRLF$
		HTML + "<tr><td colspan='2' class='code_del'>Deleted Lines of Code</td></tr>" + #CRLF$
		HTML + "<tr><td colspan='2'>&nbsp;</td></tr>" + #CRLF$
	EndIf
	HTML + "<tr><td class='code'>" + #CRLF$

	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_GetFooter(ShowDifferences)
	;Called at the end of the output, so returnvalue should be anything which will make the html perfect.
	;
	;ShowDifferences: If <> 0 user activated the Code-Difference-View.
	;                 You may alter your footer.

	HTML = ""
	If ShowDifferences
		HTML + "<tr><td colspan='2'>&nbsp;</td></tr>" + #CRLF$
		HTML + "<tr><td colspan='2' class='code_add'>Added Lines of Code</td></tr>" + #CRLF$
		HTML + "<tr><td colspan='2' class='code_del'>Deleted Lines of Code</td></tr>" + #CRLF$
	EndIf
	HTML + "</tbody></table>" + #CRLF$
	HTML + "</body></html>" + #CRLF$

	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_DelBlockStart(Anchor.s)
	;This will be the start of a block, which contains deleted lines.
	;Anchor will be used to navigate within different changes in the code.
	
	HTML = "<tr>" + #CRLF$
	HTML + "<td class='fln' colspan='2'><a name=" + #DQUOTE$ + Anchor + #DQUOTE$ + "></a></td>" + #CRLF$
	HTML + "</tr>" + #CRLF$
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_DelBlockEnd()
	;This will be the end of the above block.
	
	HTML = ""
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_DelBlockLine(Line.s)
	;One single line of code inside the block of deleted lines
	
	HTML = "<tr>" + #CRLF$
	HTML + "<td class='fln'>&nbsp;</td><td class='code_del'>" + Internal_DoHTML(Line) + "</td>" + #CRLF$
	HTML + "</tr>"
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_AddBlockStart(Anchor.s)
	;This will be the start of a block, which contains added lines.
	;Anchor will be used to navigate within different changes in the code.
	
	HTML = "<tr>" + #CRLF$
	HTML + "<td class='fln' colspan='2'><a name=" + #DQUOTE$ + Anchor + #DQUOTE$ + "></a></td>" + #CRLF$
	HTML + "</tr>" + #CRLF$
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_AddBlockEnd()
	;This will be the end of the above block.
	
	HTML = ""
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_AddBlockLine(Line.s)
	;One single line of code inside the block of added lines
	
	HTML = "<tr>" + #CRLF$
	HTML + "<td class='fln'>" + Str(LineNum) + "</td><td class='code_add'>" + Internal_DoHTML(Line) + "</td>" + #CRLF$
	HTML + "</tr>"
	LineNum + 1
	
	ProcedureReturn @HTML
EndProcedure

ProcedureDLL Template_AddLine(Line.s)
	;A single line of code
	
	HTML = "<tr>" + #CRLF$
	HTML + "<td class='fln'>" + Str(LineNum) + "</td><td class='code'>" + Internal_DoHTML(Line) + "</td>" + #CRLF$
	HTML + "</tr>" + #CRLF$
	LineNum + 1
	
	ProcedureReturn @HTML
EndProcedure
; IDE Options = PureBasic 4.61 Beta 1 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 118
; FirstLine = 81
; Folding = ---
; EnableXP
; Executable = default_with_lines.dll
; CurrentDirectory = F:\Eigene Dateien\PureBasic\
; CompileSourceDirectory
; EnableCompileCount = 104
; EnableBuildCount = 98
; EnableExeConstant